'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _glMatrix = require('gl-matrix');

var _Plane = require('./Plane');

var _Plane2 = _interopRequireDefault(_Plane);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Задаёт frustum в трёхмерном пространстве
 *
 * Взято из [three.js](https://github.com/mrdoob/three.js/blob/master/src/math/Frustum.js)
 */

var Frustum = function () {
    /**
     * @param {Array} planes Массив из шести плоскостей, формирующих frustum
     */

    function Frustum(planes) {
        _classCallCheck(this, Frustum);

        this.planes = planes;

        if (!this.planes || this.planes.length !== 6) {
            this.planes = [new _Plane2.default(), new _Plane2.default(), new _Plane2.default(), new _Plane2.default(), new _Plane2.default(), new _Plane2.default()];
        }
    }

    /**
     * Устанавливает плоскости frustum в соответствие с матрицей
     * @param {mat4} matrix
     */


    _createClass(Frustum, [{
        key: 'setFromMatrix',
        value: function setFromMatrix(matrix) {
            this.planes[0].setComponents(matrix[12] + matrix[0], matrix[13] + matrix[1], matrix[14] + matrix[2], matrix[15] + matrix[3]);

            this.planes[1].setComponents(matrix[12] - matrix[0], matrix[13] - matrix[1], matrix[14] - matrix[2], matrix[15] - matrix[3]);

            this.planes[2].setComponents(matrix[12] + matrix[4], matrix[13] + matrix[5], matrix[14] + matrix[6], matrix[15] + matrix[7]);

            this.planes[3].setComponents(matrix[12] - matrix[4], matrix[13] - matrix[5], matrix[14] - matrix[6], matrix[15] - matrix[7]);

            this.planes[4].setComponents(matrix[12] + matrix[8], matrix[13] + matrix[9], matrix[14] + matrix[10], matrix[15] + matrix[11]);

            this.planes[5].setComponents(matrix[12] - matrix[8], matrix[13] - matrix[9], matrix[14] - matrix[10], matrix[15] - matrix[11]);

            return this;
        }

        /**
         * Проверяет, находится ли {@link Box} в области frustum
         * @param {Box} box
         * @returns {Boolean}
         */

    }, {
        key: 'intersectsBox',
        value: function intersectsBox(box) {
            var p1 = _glMatrix.vec3.create();
            var p2 = _glMatrix.vec3.create();
            var planes = this.planes;

            for (var i = 0; i < 6; i++) {
                var plane = planes[i];

                p1[0] = plane.normal[0] > 0 ? box.min[0] : box.max[0];
                p2[0] = plane.normal[0] > 0 ? box.max[0] : box.min[0];
                p1[1] = plane.normal[1] > 0 ? box.min[1] : box.max[1];
                p2[1] = plane.normal[1] > 0 ? box.max[1] : box.min[1];
                p1[2] = plane.normal[2] > 0 ? box.min[2] : box.max[2];
                p2[2] = plane.normal[2] > 0 ? box.max[2] : box.min[2];

                var d1 = plane.distanceToPoint(p1);
                var d2 = plane.distanceToPoint(p2);

                // if both outside plane, no intersection
                if (d1 < 0 && d2 < 0) {
                    return false;
                }
            }

            return true;
        }
    }]);

    return Frustum;
}();

exports.default = Frustum;
module.exports = exports['default'];